package com.xhkjedu.sapi.service.notice;

import com.xhkjedu.sapi.mapper.notice.NoticeClassMapper;
import com.xhkjedu.sapi.mapper.notice.NoticeMapper;
import com.xhkjedu.sapi.mapper.notice.NoticeUserMapper;
import com.xhkjedu.sapi.model.notice.TNoticeClass;
import com.xhkjedu.sapi.model.notice.TNoticeUser;
import com.xhkjedu.sapi.mapper.notice.*;
import com.xhkjedu.sapi.model.notice.TNoticeFile;
import com.xhkjedu.sapi.model.notice.TNotice;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.sapi.vo.notice.NoticeVo;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author ywx
 * @classname NoticeService
 * @description 
 * @date 2020/6/12 17:55
 **/
@Service
public class NoticeService {
    @Resource
    private NoticeMapper noticeMapper;
    @Resource
    private NoticeClassMapper noticeClassMapper;
    @Resource
    private NoticeUserMapper noticeUserMapper;
    @Resource
    private NoticeFileMapper noticeFileMapper;

    //添加
    public Integer save(TNotice model) {
        if (N_Utils.isEmptyInteger(model.getCreatetime())) {
            model.setCreatetime(N_Utils.getSecondTimestamp());
        }
        noticeMapper.insertUseGeneratedKeys(model);
        model.setNoticeid(model.getId());
        //添加通知对象
        saveNoticeObj(model);

        //通知附件
        List<TNoticeFile> files = model.getFiles();
        if (N_Utils.isListNotEmpty(files)) {
            Integer noticeid = model.getNoticeid();
            for (TNoticeFile file : files) {
                file.setNoticeid(noticeid);
            }
            noticeFileMapper.insertList(files);
        }
        return model.getId();
    }

    //删除
    public Integer deleteById(Integer noticeid) {
        return noticeMapper.deleteByPrimaryKey(noticeid);
    }

    //修改
    public Integer update(TNotice model) {
        return noticeMapper.updateByPrimaryKeySelective(model);
    }

    //添加通知对象
    private void saveNoticeObj(TNotice model) {
        Integer noticetype = model.getNoticetype();
        if (noticetype == 1) {//通知对象1班级
            noticeClassMapper.batchSave(model);
        } else if (noticetype == 2 || noticetype == 3) {//通知对象2老师3单个学生
            model.setUsers(N_Utils.listDistinct(model.getUsers()));//通知对象去重
            noticeUserMapper.batchSave(model);
        }

        /*if (N_Utils.isListEmpty(model.getSchools())) {//兼容以前添加单个学校模式
            List<TSchool> schools = new ArrayList<>();
            TSchool s = new TSchool();
            s.setSchoolid(model.getSchoolid());
            schools.add(s);
            model.setSchools(schools);
        }
        noticeSchoolMapper.batchSave(model);//通知学校关联信息*/
    }

    //获取详情
    public TNotice findById(Integer noticeid) {
        TNotice notice = noticeMapper.findById(noticeid);
        notice.setFiles(noticeFileMapper.listByNoticeId(noticeid));
        /*Integer noticetype = notice.getNoticetype();
        if (noticetype == 1) {
            List<ClassVo> classes = noticeMapper.listClass(noticeid);
            notice.setClasses(classes);
        } else if (noticetype == 2 || noticetype == 3) {
            List<NoticeUserVo> users = noticeMapper.listUser(noticeid);
            notice.setUsers(users);
        } else {
            List<TSchool> schools = noticeMapper.listSchool(noticeid);
            notice.setSchools(schools);
        }*/
        return notice;
    }

    //获取列表
    public List <NoticeVo> listAll(TNotice notice) {
        return noticeMapper.listAll(notice);
    }

    /**
     * 用户通知列表
     * @Param [notice]
     * @Author ywx
     * @Date 2020/6/13 9:55
     * @return java.util.List<com.xhkjedu.vo.notice.NoticeVo>
     **/
    public List<NoticeVo> listMy(TNotice notice) {
        return noticeMapper.listMy(notice);
    }

    /**
     * 学生通知列表
     * @Param [notice]
     * @Author ywx
     * @Date 2020/6/22 10:02
     * @return java.util.List<com.xhkjedu.vo.notice.NoticeVo>
     **/
    public List<NoticeVo> listStudent(TNotice notice) {
        return noticeMapper.listStudent(notice);
    }

    /**
     * 用户查看
     * @Param [noticeid, createid]
     * @Author ywx
     * @Date 2020/6/13 11:21
     * @return com.xhkjedu.model.notice.TNotice
     **/
    public TNotice reade(Integer noticeid, Integer createid) {
        TNotice notice = noticeMapper.findById2(noticeid);
        notice.setFiles(noticeFileMapper.listByNoticeId(noticeid));
        noticeUserMapper.reade(noticeid, createid, N_Utils.getSecondTimestamp());
        return notice;
    }

    /**
     * 学生查看
     * @Param [noticeid]
     * @Author ywx
     * @Date 2020/6/22 11:00
     * @return com.xhkjedu.model.notice.TNotice
     **/
    public TNotice findById2(Integer noticeid) {
        return noticeMapper.findById2(noticeid);
    }

    /**
     * 用户未看通知数量
     * @Param [teacherid]
     * @Author ywx
     * @Date 2020/7/24 10:37
     * @return java.lang.Integer
     **/
    public Integer noReadNum(Integer teacherid) {
        return noticeMapper.noReadNum(teacherid);
    }

    //通知学生--网页--列表
    public List<NoticeVo> listNoticeStuWeb(TNotice notice) {
        return noticeMapper.listNoticeForStuWeb(notice);
    }

    /**
     * @Description 通知对象
     * @Date 2023/12/18 16:26
     * @Author YWX
     * @Param [noticeid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listUser(Integer noticeid) {
        return noticeUserMapper.listByNoticeId(noticeid);
    }

    /*
     * @Description 处理班级学生通知
     * @Date 2023/12/18 16:10:36
     * @Author WN
     * @Param []
     * @Return void
     **/
    public void chandleNoticeUser() {
        //获取需要处理的通知
        List<TNoticeClass> noticeClasses = noticeClassMapper.listAll();
        for (TNoticeClass noticeClass : noticeClasses) {
            List<TNoticeUser> noticeUsers = noticeClassMapper.listClassStudent(noticeClass.getClassid());
            if (N_Utils.isListNotEmpty(noticeUsers)) {
                for (TNoticeUser user : noticeUsers) {
                    user.setNoticeid(noticeClass.getNoticeid());
                    user.setReaded(1);
                    user.setCreatetime(noticeClass.getCreatetime());
                }
                noticeUserMapper.insertList(noticeUsers);
            }
        }
    }
}
